<!DOCTYPE html>
<!--
This is a starter template page. Use this page to start your new project from
scratch. This page gets rid of all links and provides the needed markup only.
-->
<html lang="zh-CN">

<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>OddAsr SLP service</title>
    <link rel="stylesheet" href="common/css/iconfont.css">
    <link rel="shortcut icon" href="images/favicon.ico" />
    <!-- Google Font: Source Sans Pro -->
    <!-- Font Awesome Icons -->
    <link rel="stylesheet" href="plugins/fontawesome-free/css/all.min.css">
    <!-- Theme style -->
    <link rel="stylesheet" href="dist/css/adminlte.min.css">
    <!-- Add icons to the links using the .nav-icon class
     with font-awesome or any other icon font library -->
    <link rel="stylesheet" href="https://code.ionicframework.com/ionicons/2.0.1/css/ionicons.min.css">

</head>

<body class="hold-transition sidebar-mini">
    <div class="wrapper">

        <!-- Navbar -->
        <nav class="main-header navbar navbar-expand navbar-white navbar-light">
            <!-- Left navbar links -->
            <ul class="navbar-nav">
                <li class="nav-item">
                    <a class="nav-link" data-widget="pushmenu" href="#" role="button" hidden="hidden"><i class="fas fa-bars"></i></a>
                </li>
            </ul>

            <!-- Right navbar links -->
            <ul class="navbar-nav ml-auto">

                <li class="nav-item">
                    <a href="/logout" role="button">
                        <i class="fas fa-sign-out-alt"></i>
                    </a>
                </li>

            </ul>
        </nav>
        <!-- /.navbar -->

        <!-- Main Sidebar Container -->
        <aside class="main-sidebar sidebar-light-primary elevation-4">
            <!-- Brand Logo -->


            <a href="index.html" class="brand-link">
                <img src="images/banner.png" style="opacity: .8">
                <!-- <img src="images/banner.png" class="elevation-4" style="opacity: .8">
                <span class="brand-text font-weight-light text-sm"> OddAsr</span> -->
            </a>

            <!-- Sidebar -->
            <div class="sidebar">
                <!-- Sidebar user panel (optional) -->
                <div class="user-panel mt-3 pb-3 mb-3 d-flex">
                    <div class="image">
                        <img src="images/header.png" class="img-circle elevation-2" alt="User Image">
                    </div>
                    <div class="info">
                        <a href="#" class="d-block">{{username}}</a>
                    </div>
                </div>

                <!-- Sidebar Menu -->
                <nav class="mt-2">
                    <ul class="nav nav-pills nav-sidebar flex-column" data-widget="treeview" role="menu" data-accordion="false">
                        
                        <li class="nav-item menu-open">
                            <a href="#" class="nav-link active">
                                <i class="nav-icon fas fa-microphone"></i>
                                <p>
                                    ASR service
                                    <i class="right fas fa-angle-left"></i>
                                </p>
                            </a>

                            <ul class="nav nav-treeview active">
                                <li class="nav-item">
                                    <a href="asr_live.html" class="nav-link active">
                                        <i class="fa fa-align-left fa-fw nav-icon active"></i>
                                        <p>Stream ASR</p>
                                    </a>
                                </li>

                                <li class="nav-item">
                                    <a href="asr_file.html" class="nav-link">
                                        <i class="fa fa-align-justify nav-icon"></i>
                                        <p>File ASR</p>
                                    </a>
                                </li>

                                <li class="nav-item">
                                    <a href="asr_sentence.html" class="nav-link">
                                        <i class="fa fa-align-center nav-icon"></i>
                                        <p>Sentence ASR</p>
                                    </a>
                                </li>

                                <!-- <li class="nav-item" >
                                    <a href="minutes_list.html" class="nav-link active">
                                        <i class="fa fa-clipboard nav-icon"></i>
                                        <p>数据管理</p>
                                    </a>
                                </li> -->
                            </ul>
                        </li>

                        <li class="nav-item">

                            <a href="#" class="nav-link">
                                <i class="nav-icon fas fa-user-cog"></i>
                                <p>
                                    My SLP
                                    <i class="right fas fa-angle-left"></i>
                                </p>
                            </a>

                            <ul class="nav nav-treeview">
                                <li class="nav-item">
                                    <a href="slp_language_model.html" class="nav-link">
                                        <i class="nav-icon fa fa-address-book"></i>
                                        <p>Language model</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="slp_acoustic_model.html" class="nav-link">
                                        <i class="nav-icon fa fa-wave-square"></i>
                                        <p>Acoustic model</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="slp_textual_substitution.html" class="nav-link">
                                        <i class="nav-icon fa fa-audio-description"></i>
                                        <p>Textual Substitution</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="hotwords.html" class="nav-link">
                                        <i class="fa fa-binoculars nav-icon"></i>
                                        <p>Hotwords</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="settings_hotwords_show.html" class="nav-link">
                                        <i class="fa fa-bookmark nav-icon"></i>
                                        <p>Hotwords list</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="sensiwords.html" class="nav-link">
                                        <i class="fa fa-briefcase nav-icon"></i>
                                        <p>Sensitive Words</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="sensiwords_show.html" class="nav-link">
                                        <i class="fa fa-bullseye nav-icon"></i>
                                        <p>Sensi-words list</p>
                                    </a>
                                </li>
                                <li class="nav-item">
                                    <a href="dialectmodal.html" class="nav-link">
                                        <i class="fa fa-certificate nav-icon"></i>
                                        <p>Dialect Model</p>
                                    </a>
                                </li>


                            </ul>
                        </li>

                        <li class="nav-item">
                            <a href="#" class="nav-link">
                                <i class="nav-icon fas fa-user"></i>
                                <p>
                                    User
                                    <i class="right fas fa-angle-left"></i>
                                </p>
                            </a>
                            <ul class="nav nav-treeview">
                                <li class="nav-item">
                                    <a href="user_pwd.html" class="nav-link">
                                        <i class="nav-icon fa fa-key"></i>
                                        <p>Modify Password</p>
                                    </a>
                                </li>
                            </ul>
                        </li>

                        <li class="nav-item">
                            <a href="#" class="nav-link ">
                                <i class="nav-icon fas fa-sliders-h"></i>
                                <p>
                                    Settings
                                    <i class="right fas fa-angle-left"></i>
                                </p>
                            </a>
                            <ul class="nav nav-treeview">
                                <li class="nav-item">
                                    <a href="settings_others.html" class="nav-link">
                                        <i class="fa fa-bars nav-icon"></i>
                                        <p>Basic Settings</p>
                                    </a>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </nav>
                <!-- /.sidebar-menu -->
            </div>
            <!-- /.sidebar -->
        </aside>

        <!-- Content Wrapper. Contains page content -->
        <div class="content-wrapper iframe-mode" data-widget="iframe" data-loading-screen="750">
            <div class="nav navbar navbar-expand navbar-white navbar-light border-bottom p-0">
                <div class="nav-item dropdown">
                    <a class="nav-link bg-danger dropdown-toggle" data-toggle="dropdown" href="#" role="button"
                        aria-haspopup="true" aria-expanded="false">Close</a>
                    <div class="dropdown-menu mt-0">
                        <a class="dropdown-item" href="#" data-widget="iframe-close" data-type="all">Close all</a>
                        <a class="dropdown-item" href="#" data-widget="iframe-close" data-type="all-other">Close other</a>
                    </div>
                </div>
                <a class="nav-link bg-light" href="#" data-widget="iframe-scrollleft"><i
                        class="fas fa-angle-double-left"></i></a>
                <ul class="navbar-nav overflow-hidden" role="tablist"></ul>
                <a class="nav-link bg-light" href="#" data-widget="iframe-scrollright"><i
                        class="fas fa-angle-double-right"></i></a>
                <a class="nav-link bg-light" href="#" data-widget="iframe-fullscreen"><i class="fas fa-expand"></i></a>
            </div>
            <div class="tab-content">
                <div class="tab-empty" style="display: block;">
                    <iframe src="asr_live.html"></iframe>
                </div>
            </div>
        </div>
        <!-- /.content-wrapper -->


        <!-- Main Footer -->
        <footer class="main-footer text-sm">
            <!-- To the right -->
            <div class="float-right d-none d-sm-inline">
                version 1.1
            </div>
            <!-- Default to the left -->
            OddAsr SLP service <strong> Copyright &copy; 2025-<span class="version_year"></span> OddMeta.</strong> All rights reserved.
        </footer>
    </div>
    <!-- ./wrapper -->

    <!-- REQUIRED SCRIPTS -->

    <!-- jQuery -->
    <script src="plugins/jquery/jquery.min.js"></script>
    <!-- Bootstrap 4 -->
    <script src="plugins/bootstrap/js/bootstrap.bundle.min.js"></script>
    <!-- AdminLTE App -->
    <script src="dist/js/adminlte.min.js"></script>
    <!--index js-->
    <script>
        $(function () {

            var myDate = new Date();
            var tYear = myDate.getFullYear();
            console.log("tYear", tYear);
            $(".version_year").html(tYear);

        });
    </script>
</body>
</html>
