.row{
    margin-left:0px;
    margin-right:0px;
}
.card-body{
    box-sizing: border-box;
}
#hot_words_type{
    font-size:14px;
    padding: 0.25rem 0.5rem;
}

.card-body.p-0 .table thead>tr>th:first-of-type {
    padding-left: 0;
    text-align:center;
}
.card-body.p-0 .table tbody>tr>td:first-of-type{
    text-align:center;
    padding-left: 0;
}

/* 添加热词页面  css */
.add_box{
   display:none;
   padding-right: 15px;
   padding-left: 15px;
   margin-top:20px;
}
.back_box {
    margin-bottom: 10px;
}
.detail_arrow_right {
    display: inline-block;
    width: 30px;
    height: 30px;
    outline: none;
    padding: 0;
    border: 0;
    background-color: transparent;
    background: url("./images/sprite_all.png") no-repeat 0px -699px;
    cursor: pointer;
    transform: rotate(180deg);
    -ms-transform: rotate(180deg);
    -moz-transform: rotate(180deg);
    -webkit-transform: rotate(180deg);
    -o-transform: rotate(180deg);
}
.control_title {
    display: inline-block;
    vertical-align: top;
    font-size: 12px;
    line-height: 30px;
    margin-left: 12px;
    font-weight: bold;
    color: #4e4e4e;
}
.add_box .right_1_adjust6 {
    width: 68px;
    height: 23px;
    float: right;
    background-color: #eef2f4;
    line-height: 23px;
    text-align: center;
    display: inline-block;
    margin-top: -4px;
    cursor: pointer;
    border: 1px solid #dcdfe1;
    color: #5f5f5f;
}

.add_box .right_1_adjust6+.btn_save {
    margin-right: 5px;
}

.btn-wrap {
    margin-bottom: 20px;
}

.btn-wrap span {
    width: 73px;
    height: 23px;
    line-height: 23px;
    display: inline-block;
    background-color: #6c757d;
    color: #fff;
    text-align: center;
    margin-right: 1px;
    cursor: pointer;
    font-size:12px;
}

.btn-wrap .current {
    background-color: #007bff;
}

.add_word_text {
    font-size:12px;
    width: 245px;
    display:inline-block;
}


.btn-add {
    width: 30px;
    height: 30px;
    background: url("./images/all2.png") no-repeat -267px -615px;
    cursor: pointer;
    display: inline-block;
    vertical-align: bottom;
}

.btn-wrap-tips, .add_word_tips {
    color: #8b8b8b;
    line-height: 40px;
    height: 40px;
    font-size:12px;
}
.hotword-text{
    display:none;
}


.add_word_area {
    padding-top: 20px;
    min-width: 520px;
    width: 50%;
    display: block;
    margin-bottom: 15px;
    color: #868686;
}

.add_word_single {
    cursor: pointer;
    border: 1px solid #dcdfe1;
    padding: 0 5px;
    height: 25px;
    margin-right: 3px;
    color: #333;
    display: inline-block;
}


.add_word_single span{
   position: relative;
   top: -4px;
   font-size: 12px;
}


.add_word_single i {
    color: #ccc;
    font-size: 18px;
    font-style: normal;
    letter-spacing: 0;
    position: relative;
    top: -3px;
}

.p0{
    padding-right: 0;
    padding-left: 0;
}

.red{
    color: #bd2130!important
}
.import-btn span{
    font-weight: 400;
}



