.form-group {
    display:flex;
    position:relative;
    margin-bottom:1.5rem;
}

.setting-item-main {
    position: relative;
    /*width: 303px;*/
    vertical-align: middle;
    display: inline;
    /* bottom: 2px; */
}
.label-text{
    width:110px;
}
.form-control{
    width:300px;
}

.showHidePassword {
    position: absolute;
    top: 14px;
    right: 5px;
    display: block;
    width: 20px;
    height: 10px;
}

.showHidePassword:hover {
    background: transparent url("./images/setting.png") -20px -784px no-repeat;
}

.hide-password {
    background: transparent url("./images/setting.png") -0px -784px no-repeat;
}

.show-password {
    background: transparent url("./images/setting.png") -20px -784px no-repeat;
}

.capitals-tip {
    position: absolute;
    top: 38px;
    left: 160px;
    background: url("./images/capitals-tip.png") no-repeat;
    width: 84px;
    height: 37px;
    padding-top: 14px;
    padding-left: 11px;
    z-index: 999;
}

.displayNone {
    display: none;
}
.tipWrapper {
    /*width: 160px;*/
    padding-left: 8px;
    position: relative;
    /*text-align: center;*/
     display: flex;
    align-items: center;
}

.tip-icon {
    display: inline-block;
    height: 16px;
    width: 16px;
    position: relative;
   /* bottom: -4px;*/
    border-radius: 50%;
    background: url("./images/setting.png");

}

.error-icon {
    background-position: -0px -763px;
}

.correct-icon {
    background-position: -16px -763px;
}
.weak-icon {
    background-position: -32px -763px;
}

.medium-icon {
    background-position: -48px -763px;
}

.strong-icon {
    background-position: -64px -763px;
}


.tip-info {
    display: inline-block;
    vertical-align: middle;
    line-height: 30px;
    height: 30px;
}

.error-tip {
    color: #d21e1e;
}

.help {
    padding-left: 10px;
    color: #3e9bd0;
    text-decoration: underline;
    vertical-align: middle;
}
.help-info {
    position: absolute;
    margin-left: 530px;
    margin-top:40px;
    background-color: #FFFFFF;
    border: 1px solid #dadada;
    width: 330px;
    /* height: 300px; */
    z-index: 100;
    padding-left: 22px;
    padding-top: 20px;
    padding-bottom: 20px;
}


.help {
    padding-left: 10px;
    color: #3e9bd0;
    text-decoration: underline;
    vertical-align: middle;
}

.help:hover{
    color:#212529;
    text-decoration: underline;
}


.help-title {
    font-weight: bold;
    text-align: left;
}

.help-li {
  /*margin-top:10px;*/
    text-align: left;
}

.help-first {
    padding-left: 24px;
}

.help-info p {
    margin-top: 10px;
}
.medium-help{
    padding-top: 4px;
}
.strong-help{
    padding-top: 4px;
}

.help-none-first {
    padding-left: 60px;
}

.capitals-tip  span , .help-info p , .tip-info , .text-tips ,.help{
    font-size:12px;
    font-weight:normal;
}

.text-tips{
    margin-left: 110px;
    position: absolute;
    top: 38px;
}